.android-console{padding:10px;height:calc(100% - 57px);overflow-y:auto}.android-console-input .ivu-input{background:#e8eaec;border-radius:0}.command-format{margin-bottom:25px;max-height:40vh;overflow-y:auto}.command-after-format{max-height:20vh;overflow-y:auto}#scroller *{overflow-anchor:none}#anchor{overflow-anchor:auto;height:1px}.android-log{padding:10px;height:calc(100% - 33px);overflow-y:auto}.device-list-cell{padding:5px 5px 5px 15px;border-bottom:1px solid #dcdee2}.start-item-info-btn{line-height:24px;vertical-align:top}.start-item-key{width:100px}.start-item-key,.start-item-type{display:inline-block;vertical-align:top;margin-left:5px}.start-item-type{width:80px}.start-item-value{width:calc(100% - 238px);display:inline-block;vertical-align:middle;margin-left:5px}.start-item-delete-btn{line-height:24px;vertical-align:top;margin-left:5px}.enable-button{cursor:pointer}.disable-button{color:#c5c8ce}.spin-icon-load{-webkit-animation:ani-demo-spin 1s linear infinite;animation:ani-demo-spin 1s linear infinite}.android-split{height:100vh;border-left:1px solid #e8eaec;border-right:1px solid #e8eaec;word-break:break-all}.android-split-pane{overflow-y:auto;height:100%}.cell-empty{position:absolute;top:40%;left:50%;-webkit-transform:translate(-50%,-50%);transform:translate(-50%,-50%);text-align:center}.device-list{height:30vh}.android-img{height:calc(70vh - 10px)}.info-header{background-color:#fff;padding:5px}.ivu-tabs>.ivu-tabs-bar{border-bottom:0;background-color:#fff;margin-bottom:0;font-weight:bolder}.ivu-tabs>.ivu-tabs-content{height:100%}#app{font-family:Avenir,Helvetica,Arial,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;color:#2c3e50}